'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash.set');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.get');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.reduce');

var _lodash6 = _interopRequireDefault(_lodash5);

var _lodash7 = require('lodash.topath');

var _lodash8 = _interopRequireDefault(_lodash7);

var _lodash9 = require('lodash.map');

var _lodash10 = _interopRequireDefault(_lodash9);

var _lodash11 = require('lodash.drop');

var _lodash12 = _interopRequireDefault(_lodash11);

var _lodash13 = require('lodash.isplainobject');

var _lodash14 = _interopRequireDefault(_lodash13);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function joinValuePath(valuePath) {
  return (0, _lodash10.default)(valuePath, function (pathSegment, index) {
    var segmentIsNumber = !isNaN(+pathSegment);

    if (segmentIsNumber) {
      return '[' + pathSegment + ']';
    } else {
      if (index === pathSegment.lenght - 1) {
        return pathSegment;
      } else {
        return pathSegment + '.';
      }
    }
  }).join('');
}

function isNumberString(string) {
  return !isNaN(+string);
}

function getFromJoiArray(schema, index) {
  return schema._inner.items[index];
}

function isArraySchema(schema) {
  return schema._type === 'array';
}

function getSubSchema(joi, schema, path) {
  if (schema.isJoi) {
    if (isNumberString(path) && isArraySchema(schema)) {
      return getFromJoiArray(schema, path);
    } else {
      return joi.reach(schema, path);
    }
  } else {
    return (0, _lodash4.default)(schema, path);
  }
}

function pickFromValuePath(joi, schema, valuePathArray) {

  if (valuePathArray.length) {
    var nextPathSegment = valuePathArray[0];

    var subSchema = getSubSchema(joi, schema, nextPathSegment);

    if (subSchema) {
      if (valuePathArray.length === 1) {
        return subSchema;
      } else {
        return pickFromValuePath(joi, subSchema, (0, _lodash12.default)(valuePathArray));
      }
    } else {
      throw {
        type: 'SchemaNotFound',
        pathArray: valuePathArray,
        schema: schema
      };
    }
  }
}

function joiPickDeep(joi, completeSchema, valuePaths) {

  return (0, _lodash6.default)(valuePaths, function (schemas, valuePath) {

    var pathSegments = (0, _lodash8.default)(valuePath);

    try {
      schemas[pathSegments[0]] = pickFromValuePath(joi, completeSchema, pathSegments);
    } catch (error) {

      if (error.type === 'SchemaNotFound') {
        var successfulPathSegmentsCount = pathSegments.length - error.pathArray.length;
        var successfulPathSegments = pathSegments.slice(0, successfulPathSegmentsCount);
        var successfulPath = joinValuePath(successfulPathSegments);

        console.warn('Could not find schema at path \'' + valuePaths + '\'. Using schema for \'' + successfulPath + ' instead.');

        schemas[pathSegments[0]] = error.schema;
      } else {
        throw error;
      }
    }

    return schemas;
  }, {});
}

exports.default = joiPickDeep;